/*
 * Copyright 2019 Regents of the University of Minnesota.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package edu.umn.nlpie.mtap;

import java.util.Arrays;

/**
 * The main class and entry points for the MTAP framework.
 * <p>
 * This object provides methods for interacting with a MTAP events service and for launching MTAP
 * processors written in Java.
 */
public final class MTAP {
  /**
   * The name used by the events service for discovery.
   */
  public static final String EVENTS_SERVICE_NAME = "mtap-events";

  /**
   * The name used by the processor services for discovery.
   */
  public static final String PROCESSOR_SERVICE_TAG = "v1-mtap-processor";

  public static void main(String[] args) {
    if (args.length > 0 && args[0].equals("events")) {
      EventsServer.main(Arrays.copyOfRange(args, 1, args.length));
    } else {
      System.out.println("Available options: \"events\"");
    }
  }
}
